package com.malk.aiwei.delegate;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.malk.aiwei.service.AWClint;
import com.malk.delegate.TBEvent;
import com.malk.server.dingtalk.DDConf;
import com.malk.server.teambition.TBConf;
import com.malk.service.teambition.TBClient;
import com.malk.utils.UtilMap;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Primary;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * OA审批事件 [主项目也有实现, 添加 @Primary 优先注入主项目实现]
 * -
 * 取消方案: 撤销和拒绝流程不继续执行连接器, 因此不使用连接器与轮询审批记录方案 [低效而且占用较高钉钉api调次数];
 * 优化方案: 通过事件订阅实现实时同步所有审批状态, 定时查询钉钉回调失败记录 [配置钉钉事件Delegate, 添加定时任务]
 */
@Slf4j
@Service
@Primary
public class TBDelegate implements TBEvent {


    @Autowired
    private TBClient tbClient;

    @Autowired
    private AWClint awClint;

    @Autowired
    private DDConf ddConf;

    @Async
    @Override
    @SneakyThrows
    public void callBackTask(JSONObject eventJson) {
        String eventName = eventJson.getString("event");
        log.info("callBackTask, {}, {}", eventName, eventJson);
        // todo 查询工作流接口
        if ("v3.task.taskflowstatus.update".equals(eventName)) {

            JSONObject data = eventJson.getJSONObject("data");
            List<String> workDones = Arrays.asList("659679ec3bf0fb46b4b68ca2", "65964fe63bf0fb46b4b6264d", "65964fe63bf0fb46b4b6264f"); // "65937802de54cc9e2351121c"; // 已完成, 触发checklist
            String approve = "65967ad70e25b16b2e91e568"; // "65937802de54cc9e23511219"; // 提交评审, 触发交付物审批
            List<String> allState = new ArrayList<>();
            allState.addAll(workDones);
            allState.add(approve);
            if (!allState.contains(data.getString("tfsId"))) {
                return;
            }
            List<Map> rTask = tbClient.queryTaskDetail(data.getString("taskId"), null, null);
            List<Map> customfields = UtilMap.getList(rTask.get(0), "customfields");
            String taskCode = "658d21e84aa556a15b887402"; // "659a67b83207833e1e2f744e"; // 任务编码字段ID
            String tCode = TBConf.getTaskFieldValue_First(customfields, taskCode);
            // 过滤编码为空任务
            if (StringUtils.isBlank(tCode)) {
                return;
            }
            // 匹配发起人, 新版本id映射表解析
            Map<String, String> extra = (Map) tbClient.idMapQuery(data.getString("creatorId"), "dingTalk-user", ddConf.getCorpId()).get(0).get("extra");
            String userId = extra.get("userId");
            if (workDones.contains(data.getString("tfsId"))) {
                String result = awClint.check(tCode, data.getString("projectId"), data.getString("taskId"), data.getString("creatorId"), userId);
                if (StringUtils.isBlank(result)) {
                    return;
                }
                // 回写审批记录链接, 自定义字段 [共用错误原因字段]
                Map body = TBConf.assembleCustomFieldName("checklist状态", "检查中");
                tbClient.updateTaskCustomField(data.getString("taskId"), data.getString("creatorId"), body);
                Map body2 = TBConf.assembleCustomFieldName("自检要素", result);
                tbClient.updateTaskCustomField(data.getString("taskId"), data.getString("creatorId"), body2);
            }
            if (approve.equals(data.getString("tfsId"))) {
                String deliverable = "65694e28b2fdeecdb23b5a2f"; // "659655758c54dca4b61c274e"; // 交付物任务字段ID
                List<Map> attachments = TBConf.getTaskFieldValue(customfields, deliverable).stream().map(item -> {
                    Map link = (Map) JSON.parse(String.valueOf(item.get("metaString")));
                    // ppExt: 知识库绑定没有文件后缀, 宜搭识别目前仅能点击下载跳转预览, 添加统一docx后缀, 文档会自行区分
                    return UtilMap.map("downloadUrl, name", link.get("url"), link.get("title") + ".docx");
                }).collect(Collectors.toList());

                String result = awClint.approve(tCode, data.getString("projectId"), attachments, data.getString("taskId"), data.getString("creatorId"), userId);
                if (StringUtils.isBlank(result)) {
                    return;
                }
                Map body2 = TBConf.assembleCustomFieldName("交付件描述", result);
                tbClient.updateTaskCustomField(data.getString("taskId"), data.getString("creatorId"), body2);
            }
        }
    }

    @Async
    @Override
    @SneakyThrows
    public void callBackProject(JSONObject eventJson) {
        String eventName = eventJson.getString("event");
        log.info("callBackProject, {}", eventJson);
    }

}
